import * as React from 'react';
import {
  TEXT_RESOURCE_EDITOR_ID,
  BINARY_DIFF_EDITOR_ID,
  SIDE_BY_SIDE_EDITOR_ID,
  TEXT_DIFF_EDITOR_ID,
} from '../constants';
import {
  MonacoEditor,
  MonacoDiffEditor,
} from '@workbench-stack/components';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { IEditorRegistry } from '@workbench-stack/core';
import { ClientExtensions, CacheTypenames } from '../constants';
import { EditorContributionFill, IEditorTextProps, IEditorDiffTextProps } from './editor';

export interface IEditorContributionProps {
  ID: string;
}



// String Editor
export const TextResourceEditor: React.FC<IEditorContributionProps> = props => {

  return (
    <EditorContributionFill >
      {(editorProps: IEditorTextProps) => {
        return editorProps.visibleEditorId === props.ID ? (
          <MonacoEditor {...editorProps} />
        ) : null;
      }}
    </EditorContributionFill>
  );
};
TextResourceEditor.defaultProps = { ID: TEXT_RESOURCE_EDITOR_ID};
/**
 * Text diff editor id.
 */
export const TextDiffEditor: React.FC<IEditorContributionProps> = props => {
  return (
    <EditorContributionFill>
      {(editorProps: IEditorDiffTextProps) =>
        editorProps.visibleEditorId === props.ID ? (
          <MonacoDiffEditor {...editorProps} />
        ) : null
      }
    </EditorContributionFill>
  );
};
TextDiffEditor.defaultProps = { ID: TEXT_DIFF_EDITOR_ID };


/**
 * Binary diff editor.
 */
export const BinaryResourceDiffEditor: React.FC<IEditorContributionProps> = props => {

  return (
    <EditorContributionFill>
      {(editorProps: IEditorDiffTextProps) =>
        editorProps.visibleEditorId === props.ID ? (
          <MonacoDiffEditor {...editorProps} />
        ) : null
      }
    </EditorContributionFill>
  );
};
BinaryResourceDiffEditor.defaultProps = { ID: BINARY_DIFF_EDITOR_ID };

export const SideBySideEditor: React.FC<IEditorContributionProps> = props => {

    return (
      <EditorContributionFill>
        {(editorProps: IEditorDiffTextProps) =>
          editorProps.visibleEditorId === props.ID ? (
            <MonacoDiffEditor {...editorProps} />
          ) : null
        }
      </EditorContributionFill>
    );
};
SideBySideEditor.defaultProps = { ID: SIDE_BY_SIDE_EDITOR_ID };



export const EditorContribution = props => {
  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: TextResourceEditor.defaultProps.ID,
    name: TextResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.ResourceEditorInput }, { __typename: CacheTypenames.UntitledEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: TextResourceEditor.defaultProps.ID,
    name: TextResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.FileEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: TextDiffEditor.defaultProps.ID,
    name: TextResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.DiffEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: BinaryResourceDiffEditor.defaultProps.ID,
    name: TextResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.DiffEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: SideBySideEditor.defaultProps.ID,
    name: TextResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.SideBySideEditorInput } ]);

  return (
    <>
      <TextResourceEditor {...props} />
      <TextDiffEditor {...props} />
      <BinaryResourceDiffEditor {...props} />
      <SideBySideEditor {...props} />
    </>
  );
};
