
import * as React from 'react';
import { Slot, IMonacoEditorProps, IMonacoEditorTextProps, IMonacoDiffEditorProps, createSlotFill } from '@workbench-stack/components';
import { EDITOR_SLOT_PROVIDER } from '../constants';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { IEditorRegistry, IEditorInput } from '@workbench-stack/core';
import { ClientExtensions, CacheTypenames } from '../constants';



export type IEditorComponentProps = { input: IEditorInput, visibleEditorId: string };
export type IEditorTextProps = IMonacoEditorTextProps  & IEditorComponentProps;
export type IEditorDiffTextProps = IMonacoDiffEditorProps & IEditorComponentProps;

export type IEditorProps = IEditorTextProps | IEditorDiffTextProps;

type IEditorSlotProps = Exclude<IMonacoEditorProps, 'uri'> & { input: IEditorInput };

export const { Fill: EditorContributionFill, Slot: EditorSlot }  = createSlotFill<IEditorProps>(EDITOR_SLOT_PROVIDER);

export const Editor: React.FC<IEditorSlotProps> = (props) => {
    const registry = Registry.as<IEditorRegistry>(ClientExtensions.Editors);
    const typename = props.input ? props.input.__typename : CacheTypenames.FileEditorInput;
    const editorDescriptor = registry.getEditor({ __typename: typename});


    return (
        editorDescriptor ? <EditorSlot fillProps={{...props, visibleEditorId: editorDescriptor.id, uri: props.input.resource as any }} /> : null
    );
};




