import gql from 'graphql-tag';
import { CacheTypenames } from '../../constants';
import { IResolvers, AllContextFragmentDoc, IAllContextFragment  } from '@workbench-stack/core';

const EDITOR_CONTEXT_FRAGMENT = gql`
    fragment context on Context {
        resource
        resourceSet
        resourceScheme
        resourceLangId
        resourceExtName
        resourceFilename
        isFileSystemResource
    }
`;

const resolvers: IResolvers = {
    Query: {
        getContextProperty(_, {  }, { cache, getCacheKey }) {
            try {
              const data = cache.readFragment({
                fragment: EDITOR_CONTEXT_FRAGMENT,
                  id: getCacheKey({ __typename: CacheTypenames.Context }),
              });
              console.log('[ha5sb]: ', { data, cache });
              return data;
            } catch (err) {
              console.log('[ha5sb]: ', { error: err });
            }
        },
        getContextData(parent, args, context) {
            return context.cache.readFragment<IAllContextFragment>({
                fragment: AllContextFragmentDoc,
                id: this.utility.getCacheKey({ __typename: CacheTypenames.Context }),
            });
        },
        selectedResource: (parent, args, context) => {
            return context.client.container.resourceContextKey.get() as any;
        },

    },
    Mutation: {
        updateContext: (_, args, context) => {
            context.client.container.contextKeyService.setContext(args.key, args.value);
            return true;
        },
        updateSelectedResource: (parents, { resource }, context ) => {
              context.client.container.resourceContextKey.set(resource);
              return true;
        },
    },
};

export { resolvers };
